import * as React from 'react';
import classNames from "classnames";

import {ProgressCircle} from '@ladifire-ui-react/progress';
import {DynamicList, DynamicListProps} from './DynamicList';
import gs from "@ladifire-css/sass/index.scss";
import {ListItem} from "./ListItem";

const C = 20;

export interface MessageDynamicListProps extends DynamicListProps {

}

export interface MessageDynamicListState {

}

export class MessageDynamicList extends DynamicList<DynamicListProps, MessageDynamicListState> {
  static displayName = 'DynamicList';
  private pendingUpdate: boolean | null;
  private overscan: number;
  private startTop: number;
  private endBottom: number;
  private scrollAnimation: any;
  startKey: any;
  endKey: any;
  constructor( props: DynamicListProps ) {
    super( props );
    this.pendingUpdate = null;
    this.recomputeOffsets( props );
    this.overscan = Math.max( 200, props.height / 2 );
  }

  componentDidUpdate() {
    super.componentDidUpdate();
    0 === this.props.keys.length && this.loadMore();
    this.layout.setBottomMargin( this.getLoadingOffsetBottom() );
    this.pendingUpdate = false;
  }

  UNSAFE_componentWillReceiveProps(r: DynamicListProps) {
    super.UNSAFE_componentWillReceiveProps(r);
    if (r.itemToScroll !== this.props.itemToScroll) {
      this.scrollToKey(r.itemToScroll)
    }
  }

  getOffset() {
    const t = this.getLoadingOffsetTop();
    return this.startTop - t;
  }

  getLoadingOffsetTop() {
    // return 0;

    if (!this.props.keys.length) return 0;
    if (this.props.reachedStart && 0 === this.state.start) return 0;
    if (this.layout.getTotalHeight() <= this.props.height) return 0;
    if (0 !== this.state.start) return 40;
    return 40
  }

  getLoadingOffsetBottom() {
    // return 0;
    if ( ! this.props.keys.length ) return 0;
    if ( this.props.reachedEnd && this.state.end === this.props.keys.length ) return 0;
    return 40;
  }

  getScrollTop() {
    if (this.props.useWindowScroller) {
      return this.props.scrollTop;
    }
    if ( 'number' === typeof this.scrollTop ) return this.scrollTop;
    if ( 'number' === typeof this.pendingScrollTop ) return this.pendingScrollTop;
    return super.getScrollTop() + this.getOffset();
  }

  getContentHeight() {
    const a = this.layout.getTotalHeight();
    if (this.props.useWindowScroller) {
      return a - this.getOffset() + this.getLoadingOffsetBottom()
    }
    return this.endBottom - this.getOffset() + this.getLoadingOffsetBottom();
  }

  getTop( t: string ) {
    return this.layout.getTop( t ) - this.getOffset();
  }

  getBottom( t: string ) {
    return this.layout.getBottom( t ) - this.getOffset();
  }

  setScrollTop( a: number ) {
    const r = a < this.getOffset();
    const n = a + this.props.height > this.endBottom + this.getLoadingOffsetBottom();
    if ( ( r && 0 !== this.state.start ) || ( n && this.state.end !== this.props.keys.length ) ) {
      this.scrollTop = a;
      this.relayout();
      return;
    }

    super.setScrollTop( a - this.getOffset() );
  }

  scrollToOffset( a: number ) {
    const r = arguments.length > 1 && void 0 !== arguments[ 1 ] ? arguments[ 1 ] : {},
      n = r.animate,
      i = r.absolute;
    if ( ! this.node ) return;
    if ( this.pendingUpdate ) return;
    let s = i ? a : a - this.getOffset();
    ! i && s > 0 && s < this.getLoadingOffsetTop() && ( s = 0 );
    const o = this.bracketScrollTop( s );
    if ( ! i && o !== s ) {
      this.setScrollTop( a );
      return;
    }
    if ( n ) {
      this.scrollAnimation = this.node.scrollWithAnimation( o );
      return;
    }

    super.setScrollTop( o );
  }

  bracketScrollTop( t: number ) {
    return Math.max( 0, Math.min( this.getContentHeight() - this.props.height, t ) );
  }

  resetRendered() {
    this.overscan = this.props.height / 2;
  }

  loadMore(start: number = this.state.start, end: number = this.state.end, props = this.props) {
    const t = start; //arguments.length > 0 && void 0 !== arguments[ 0 ] ? arguments[ 0 ] : this.state.start;
    const a = end; //arguments.length > 1 && void 0 !== arguments[ 1 ] ? arguments[ 1 ] : this.state.end;
    const r = props; //arguments.length > 2 && void 0 !== arguments[ 2 ] ? arguments[ 2 ] : this.props;
    const n =
      arguments.length > 3 && void 0 !== arguments[ 3 ] ? arguments[ 3 ] : this.getScrollTop();
    const i = this.layout.getTotalHeight();
    const s = 0 === t && n < 1;
    const o = a === r.keys.length && n + r.height > i - 1;
    const l = ! r.keys.length || i <= r.height || s;
    const c = ! r.keys.length || i <= r.height || o;

    // console.log('===scrollTop: ', this.props.scrollTop);
    // console.log('===getTotalHeight: ', this.layout.getTotalHeight());
    // console.log('===containerHeight: ', this.props.height);

    const _mustRequestMore = this.props.useWindowScroller && this.props.scrollTop + this.props.height > i + 40;

    if ( _mustRequestMore && typeof r.loadPost === 'function' ) {
      return r.loadPost();
    }

    // console.log('mustCallLoadPost: ', s, o);

    if ( l && c && typeof r.loadAround === 'function' ) {
      return r.loadAround();
    }
    if ( l && typeof r.loadPre === 'function' ) {
      return r.loadPre();
    }
    if ( (c /* || mustCallLoadPost*/) && typeof r.loadPost === 'function' ) {
      return r.loadPost();
    }
    return false;
  }

  relayout() {
    const t = this;
    this.setState( function( e: any, a: any ) {
      if ( a.isLoading ) return null;
      const r = e.start,
        n = e.end;
      const i = t.startKey;
      const o = t.endKey;
      t.recomputeOffsets( a, r, n );
      const l = this.props.useWindowScroller ? this.props.scrollTop : t.node ? t.node.scrollTop() : 0;
      const c = t.getScrollTop();
      let u = l - 1 <= 0 || c - 1 <= t.getOffset();
      let d = l + 1 >= t.getContentHeight() - a.height || c + 1 >= t.endBottom - a.height;
      let p = {
        start: r,
        end: n,
      };

      if (
        t.startKey !== i ||
        t.endKey !== o ||
        ( u && 0 !== r ) ||
        ( d && n !== a.keys.length )
      ) {
        e.scrolling && ( t.overscan = 8e3 );
        p = t.computeBounds( a, c );
        const m = p,
          h = m.start,
          f = m.end;
        t.recomputeOffsets( a, h, f );
        u = c - 1 <= t.getOffset();
        d = c + a.height + 1 >= t.endBottom + t.getLoadingOffsetBottom();
      }
      // if ( ( u && 0 === p.start ) ||
      //     ( d && p.end === a.keys.length ) ||
      //     ( t.layout.getTotalHeight() < a.height && 0 === p.start ) ) {
      //   // console.log('must call load more...');
      //   t.loadMore( r, n, a );
      // }
      const v = t.layout.getTops();

      if ( p.start === r && p.end === n && v === e.tops ) return null;
      t.scrollAnimation && t.scrollAnimation.cancel();
      t.pendingUpdate = true;
      // console.log("c", c);
      c > 0 && ( t.scrollTop = c );
      return Object.assign( {}, p, {
        tops: v,
      } );
    } );
  }

  recomputeOffsets(props?: MessageDynamicListProps, xx?: any, y?: any) {
    const t = arguments.length > 0 && void 0 !== arguments[ 0 ] ? arguments[ 0 ] : this.props;
    const a = arguments.length > 1 && void 0 !== arguments[ 1 ] ? arguments[ 1 ] : this.state.start;
    const r = arguments.length > 2 && void 0 !== arguments[ 2 ] ? arguments[ 2 ] : this.state.end;
    const n = this.layout.getTotalHeight() <= t.height;
    this.startKey = t.keys[ a ];
    this.endKey = t.keys[ r - 1 ];

    this.startTop = n ? 0 : this.layout.getTop( this.startKey ) || 0;
    this.endBottom = this.layout.getBottom( this.endKey ) || 0;
  }

  computeBounds(argument1?: any, argument2?: any) {
    const a = arguments.length > 0 && void 0 !== arguments[ 0 ] ? arguments[ 0 ] : this.props;
    const r = arguments[ 1 ];
    const n = r - this.overscan;
    const i = r + a.height + this.overscan;
    const s = super.getBounds( n, i );
    return this.snapBounds( a, s );
  }

  snapBounds(argument1?: any, argument2?: any) {
    const t = arguments.length > 0 && void 0 !== arguments[ 0 ] ? arguments[ 0 ] : this.props;
    const a = arguments[ 1 ];
    const r = a.start,
      n = a.end;

    return {
      start: r < C ? 0 : r,
      end: t.keys.length - n < C ? t.keys.length : n,
    };
  }

  renderLoading( t: any ) {
    const a: React.CSSProperties = t
      ? {
        top: 0,
        paddingBottom: 8,
        paddingTop: 8,
        position: 'absolute',
      }
      : {
        bottom: 0,
        paddingBottom: 8,
        paddingTop: 8,
        position: 'absolute',
      };

    const r = t ? 'degradedLoadingTop' : 'degradedLoadingBottom';

    return (
      <div
        key={r}
        className={classNames(
          gs.display__flex,
          gs.width__100_percent,
          gs.align_item__center,
          gs.justify_content__center,
          gs.background_color__transparent,
        )}
        style={a}
      >
        {
          typeof this.props.itemLoadingRenderer === 'function' ? this.props.itemLoadingRenderer() : (
            <div
              className={classNames(
                gs.display__flex,
                gs.align_item__center,
                gs.justify_content__center,
              )}
            >
              {
                typeof this.props.itemLoadingRenderer === 'function' ? this.props.itemLoadingRenderer() : (
                  <ProgressCircle
                    aria-label="loading"
                    isIndeterminate={true}
                    size={24}
                  />
                )
              }
            </div>
          )
        }
      </div>
    );

    // return React.createElement(
    //   'div',
    //   {
    //     key: r,
    //     className: classNames(
    //       gs.display__flex,
    //       gs.width__100_percent,
    //       gs.align_item__center,
    //       gs.justify_content__center,
    //       gs.background_color__transparent,
    //     ),
    //     style: a,
    //   },
    //   React.createElement(
    //     'div',
    //     {
    //       className: 'p-degraded_list__loading_txt',
    //     },
    //     'Đang tải thêm...'
    //   )
    // );
  }

  renderEmptyState = () => {
    return (
      <div
        className={classNames(
          gs.display__flex,
          gs.align_item__center,
          gs.justify_content__center,
        )}
        style={{
          width: this.props.width,
          height: this.props.height,
        }}
      >
        {
          this.props.isLoading || this.props.isLoadingMore ? (
            <ProgressCircle
              aria-label="loading"
              isIndeterminate={true}
              size={24}
            />
          ) : (
            <div/>
          )
        }
      </div>
    );
  };

  renderItem( t: string, r: any ) {
    if (!this.props.useWindowScroller) {
      return super.renderItem(t, r);
    }

    const a = this.layout.getTop( t );
    const n = this.layout.getHeight( t );
    const i = this.layout.getHeightValidity( t );
    const s = this.state.activeItem === t;
    const o =
      null === this.state.activeItem &&
      this.state.hasKeyboardFocus &&
      this.state.initialActiveItem === t;
    const l = this.state.mouseDownTargetItem === t;
    const c = this.state.hoveredItem === t;
    const u = this.props.getAriaLabelForRow;
    // const p = this.props.isItemAriaExpanded;
    const d =
      u &&
      u( r, {
        hasFocus: s,
      } );
    // const h = p && p( t, a );
    return React.createElement(
      ListItem,
      {
        key: t,
        id: t,
        height: n,
        validity: i,
        style: {
          top: a,
        },
        role: this.props.itemRole,
        hasFocus: s,
        hasFocusWithin: o,
        isMouseDownOnItem: l,
        onHeightChange: this.setHeight,
        onFocusEnter: this.onItemFocusEnter,
        onFocusWithin: this.onItemFocusWithin,
        onMouseOver: this.onItemMouseOver,
        onMouseLeave: this.onItemMouseLeave,
        onMouseDown: this.onItemMouseDown,
        onMouseUp: this.onItemMouseUp,
        ariaLabel: d,
      },
      this.props.rowRenderer( r, {
        hasFocus: s,
        isHovered: c,
      } )
    );
  }

  renderRows = () => {
    const r = this;
    const t = this.props,
      n = t.keys,
      a = t.persistentKeys;
    const s = this.state,
      l = s.start,
      i = s.end;
    const o = [];

    for ( let u = l; u < i; u++ ) {
      const c = n[ u ];
      c && o.push( this.renderItem( c, u ) );
    }
    a && a.forEach( function( e: any ) {
      const tt = n.indexOf( e );
      if ( tt < 0 ) return;
      if ( tt >= l && tt < i ) return;
      o.push( r.renderItem( e, tt ) );
    } );

    return o;
  };

  // @ts-ignore
  renderItems() {
    if (!this.props.keys || this.props.keys.length === 0) {
      return this.renderEmptyState();
    }

    const a = this.props.useWindowScroller ? this.renderRows() : super.renderItems(false);
    const shouldRenderLoadingPre = this.props.keys.length > 1 && ( ! this.props.reachedStart || this.state.start > 0 );
    const shouldRenderLoadingPost =
      this.props.keys.length > 1 &&
      ( ! this.props.reachedEnd || this.state.end < this.props.keys.length );

    // TODO: check this
    // @ts-ignore
    return [ shouldRenderLoadingPre && this.renderLoading( true ) ].concat( a, [
      shouldRenderLoadingPost && this.renderLoading( false ),
    ] );
  }

  renderContents() {
    if (!this.props.useWindowScroller) {
      return super.renderContents();
    }

    const r = this.getContentsClassName();
    let t = this.layout.getTotalHeight();
    const n = this.renderItems();

    const shouldRenderLoadingPre = this.props.keys.length > 1 && ( ! this.props.reachedStart || this.state.start > 0 );
    const shouldRenderLoadingPost =
      this.props.keys.length > 1 &&
      ( ! this.props.reachedEnd || this.state.end < this.props.keys.length );

    if (shouldRenderLoadingPost) {
      t = t + 40;
    }

    return (
      <div
        ref={this.setContentRef}
        className={r}
        role='presentation'
        style={{
          position: 'relative',
          height: t,
        }}
      >
        {n}
      </div>
    );
  }
}
